/*
 *
 * Copyright (C) 2020 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

// TPS8802 Smoke with Load Switch Measurement Sequence for SLVAET3 V1
// "Designing a 10-Year Battery Life Smoke Alarm with the TPS8802"
// Written in Energia 1.8.11E23 for MSP-EXP430F5529LP
// Launchpad pin map: https://energia.nu/pinmaps/img/MSP-EXP430F5529LP.jpeg
// Use the schematics in SLVAET3 with the following connections to a TPS8802EVM:
// Pin 2 (J1) on MSP-EXP430F5529LP to LEDEN
// Pin 3 (J1) on MSP-EXP430F5529LP to GPIO
// Pin 4 (J1) on MSP-EXP430F5529LP to load switch enable pin
// Pin 6 (J1) on MSP-EXP430F5529LP to AMUX
// Pin 9 (J1) on MSP-EXP430F5529LP to SCL
// Pub 10(J1) on MSP-EXP430F5529LP to SDA
// Disconnect USB and apply 2.2V power to MSP-EXP430F5529LP 3V3 terminal after programming
// This lowers the digital high threshold voltage for I2C communication


#include <Wire.h>

byte addr = 0x3F; // TPS8802 device address with CSEL=GND
byte LEDENpin=2; // LEDEN pin connection on Launchpad
byte GPIOpin=3; // GPIO pin connection on Launchpad
byte loadSWpin=4; // Load switch enable pin connection on Launchpad
byte AMUXpin=6; // AMUX pin connection on Launchpad
int smokeval=0; // initialize smoke measurement value
int floorval=0; // initialize ambient light measurement value

void setup()
{
  Wire.begin(); // enable I2C communication
  Wire.setClock(400000); // Set I2C speed to 400kHz
  pinMode(GPIOpin,INPUT); // configure Launchpad GPIO pin as input
  pinMode(LEDENpin,OUTPUT); // configure Launchpad LEDEN pin as output
  pinMode(loadSWpin,OUTPUT); // Configure Launchpad load switch enable pin as output
}

void loop()
{
  digitalWrite(loadSWpin,1); // enable load switch
  delay(8); //wait 8ms for initialization
  delayMicroseconds(500); //wait 500µs more for initialization
  I2Cwrite2(addr,0x10,0x1F,0x31); // write TEMPCOA=11, PREF_SEL=1, PGAIN=11. PDACA=0x31 for 100mA current
  I2Cwrite2(addr,0x06,0x0C,0x04); //write BST_EN=0, PAMP_EN=1 and PGAIN_EN=1. LEDPIN_EN=1
  I2Cwrite(addr,0x0B,0x22); //write AMUX_SEL=AOUT_PH (photo gain), GPIO=STATUS1
  floorval=analogRead(AMUXpin); //read the ambient light level
  digitalWrite(LEDENpin,1); //enable photo LED
  smokeval=analogRead(AMUXpin); //read the LED signal level
  digitalWrite(LEDENpin,0); // disable photo LED
  digitalWrite(loadSWpin,0); // disable the load switch

  // process collected data here (not implemented)

  delay(3323); // wait 3.3 seconds before taking the next measurement
}

void I2Cwrite(byte addr1, byte addr2, byte val)
{
  Wire.beginTransmission(addr1);
  Wire.write(addr2);
  Wire.write(val);
  Wire.endTransmission();
}

void I2Cwrite2(byte addr1, byte addr2, byte val1, byte val2)
{
  Wire.beginTransmission(addr1);
  Wire.write(addr2);
  Wire.write(val1);
  Wire.write(val2);
  Wire.endTransmission();
}

byte I2Cread(byte addr1, byte addr2) {
  Wire.beginTransmission(addr1);
  Wire.write(addr2);
  Wire.endTransmission();
  Wire.requestFrom(addr1, 1);
  while (Wire.available() == 0);
  return Wire.read();
}
